-- Script that implements a specialization to erase decorations 
-- such as bushes and other types of vegetation when a mulcher is operating
-- Authors: SMI Modding Team (Nicola)
-- Game version: Farming Simulator 22
-- Version: 1.0.0.0 - 22.07.2024	
RealMulcher = {};

function RealMulcher.prerequisitesPresent(specializations)
	return true
end

function RealMulcher.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "processMulcherArea", RealMulcher.processMulcherArea)
end

function RealMulcher:onLoad(...)
	self.spec_realMulcher = {}

	local initialized = false
	if self.spec_workArea.workAreas ~= nil then
		for _, workArea in pairs(self.spec_workArea.workAreas) do
			if workArea.type == WorkAreaType.MULCHER then
				initialized = true
				workArea.requiresOwnedFarmland = false	-- questo gestisce già il poter tagliare ovunque, rimangono poi solo le decorazioni
			end
		end
	end
	if not initialized then
		print("Error: RealMulcher initialization failed for " .. tostring(self:getFullName()) .. " !")
		return
	end
end

function RealMulcher:processMulcherArea(superFunc, workArea, dt)
    local spec = self.spec_mulcher

    spec.isWorkingIdle = self:getLastSpeed() > 0.5

    local xs,_,zs = getWorldTranslation(workArea.start)
    local xw,_,zw = getWorldTranslation(workArea.width)
    local xh,_,zh = getWorldTranslation(workArea.height)

	FSDensityMapUtil.clearDecoArea(xs, zs, xw, zw, xh, zh) -- togli le decorazioni
    local realArea, area = FSDensityMapUtil.updateMulcherArea(xs,zs, xw,zw, xh,zh)
    if realArea > 0 and spec.isWorkingIdle then
        local effects = spec.workAreaToEffects[workArea.index]
        if effects ~= nil then
            for _, effect in ipairs(effects) do
                if effect.activeDirection == 0 or self.movingDirection == effect.activeDirection then
                    effect.activeTime = g_currentMission.time + effect.activeTimeDuration

                    if not effect.isActive then
                        g_effectManager:setFillType(effect.effects, spec.effectFillType)
                        g_effectManager:startEffects(effect.effects)

                        effect.isActive = true
                        self:raiseDirtyFlags(spec.effectDirtyFlag)
                    end
                end
            end
        end

        spec.lastWorkTime = g_time
    end

    FSDensityMapUtil.eraseTireTrack(xs,zs, xw,zw, xh,zh)

    spec.isWorking = (g_time - spec.lastWorkTime) < 500

    if spec.isWorking then
        spec.stoneLastState = FSDensityMapUtil.getStoneArea(xs, zs, xw, zw, xh, zh)
    else
        spec.stoneLastState = 0
    end

    return realArea, area
end
